#include "graph.h"

Graph::Graph(QWidget *parent) : QWidget(parent)
{

}

Graph::~Graph()
{
	for(NodeSet::Iterator it = nodeSet.begin(); it != nodeSet.end(); it++)
		delete (*it);
	for(EdgeSet::Iterator it = edgeSet.begin(); it != edgeSet.end(); it++)
		delete (*it);
}

bool Graph::addNode(Node * node)
{
	if(node == NULL) return false;

	for(QSet<QString>::iterator it = node->hashtags.begin(); it != node->hashtags.end(); it++)
		hashtags.insert((*it));

	NodeSet::Iterator it = nodeSet.find(node->id);
	if(it == nodeSet.end())
	{
		nodeSet.insert(node->id, node);
		return true;
	}
	else
	{
		(*it)->weight++;
		for(QSet<QString>::iterator hash_it = node->hashtags.begin(); hash_it != node->hashtags.end(); hash_it++)
			(*it)->hashtags.insert((*hash_it));
		delete node;
		return false;
	}
}

bool Graph::addEdge(Edge * edge)
{
	if(edge == NULL) return false;

	if(nodeSet.find(edge->source) != nodeSet.end() && nodeSet.find(edge->target) != nodeSet.end())
	{
		if(edgeSet.find(qMakePair(edge->source, edge->target)) == edgeSet.end())
		{
			edgeSet.insert(qMakePair(edge->source, edge->target), edge);
			return true;
		}
		else
		{
			delete edge;
			return false;
		}
	}
	else
	{
		delete edge;
		return false;
	}
}

int Graph::nodesNumber()
{
	return nodeSet.size();
}

int Graph::edgesNumber()
{
	return edgeSet.size();
}

void Graph::makeXml(QString path)
{
	QDomDocument xml;
	QDomProcessingInstruction xmlDeclaration = xml.createProcessingInstruction("xml", "version=\"1.0\" encoding=\"UTF-8\"");
	xml.appendChild(xmlDeclaration);
	QDomElement gexf = xml.createElement("gexf");
		QDomAttr xmlns = xml.createAttribute("xmlns");
			xmlns.setValue("http://www.gexf.net/1.2draft");
			gexf.setAttributeNode(xmlns);
		QDomAttr version = xml.createAttribute("version");
			version.setValue("1.2");
			gexf.setAttributeNode(version);
	xml.appendChild(gexf);
		QDomElement meta = xml.createElement("meta");
			QDomAttr lastmodifieddate = xml.createAttribute("lastmodifieddate");
				lastmodifieddate.setValue(QDateTime::currentDateTime().toString("yyyy-MM-dd"));
				meta.setAttributeNode(lastmodifieddate);
			QDomElement creator = xml.createElement("creator");
				creator.appendChild(xml.createTextNode("H.Croubois"));
				meta.appendChild(creator);
			QDomElement description = xml.createElement("description");
				description.appendChild(xml.createTextNode("Produced using QtTwitGraph"));
				meta.appendChild(description);
		gexf.appendChild(meta);
		QDomElement graph = xml.createElement("graph");
			QDomAttr mode = xml.createAttribute("mode");
				mode.setValue("dynamic");
				graph.setAttributeNode(mode);
			QDomAttr defaultedgetype = xml.createAttribute("defaultedgetype");
				defaultedgetype.setValue("directed");
				graph.setAttributeNode(defaultedgetype);
			QDomAttr timeformat = xml.createAttribute("timeformat");
				timeformat.setValue("date");
				graph.setAttributeNode(timeformat);
			QDomElement attributes = xml.createElement("attributes");
				QDomAttr attributes_attr = xml.createAttribute("class");
					attributes_attr.setValue("node");
					attributes.setAttributeNode(attributes_attr);
				QDomElement attribute1 = xml.createElement("attribute");
					QDomAttr attribute1_attr1 = xml.createAttribute("id");
						attribute1_attr1.setValue("name");
						attribute1.setAttributeNode(attribute1_attr1);
					QDomAttr attribute1_attr2 = xml.createAttribute("title");
						attribute1_attr2.setValue("Name");
						attribute1.setAttributeNode(attribute1_attr2);
					QDomAttr attribute1_attr3 = xml.createAttribute("type");
						attribute1_attr3.setValue("string");
						attribute1.setAttributeNode(attribute1_attr3);
					attributes.appendChild(attribute1);
				QDomElement attribute2 = xml.createElement("attribute");
					QDomAttr attribute2_attr1 = xml.createAttribute("id");
						attribute2_attr1.setValue("screen_name");
						attribute2.setAttributeNode(attribute2_attr1);
					QDomAttr attribute2_attr2 = xml.createAttribute("title");
						attribute2_attr2.setValue("Screen Name");
						attribute2.setAttributeNode(attribute2_attr2);
					QDomAttr attribute2_attr3 = xml.createAttribute("type");
						attribute2_attr3.setValue("string");
						attribute2.setAttributeNode(attribute2_attr3);
					attributes.appendChild(attribute2);
				QDomElement attribute3 = xml.createElement("attribute");
					QDomAttr attribute3_attr1 = xml.createAttribute("id");
						attribute3_attr1.setValue("tweet_number");
						attribute3.setAttributeNode(attribute3_attr1);
					QDomAttr attribute3_attr2 = xml.createAttribute("title");
						attribute3_attr2.setValue("Tweet Number");
						attribute3.setAttributeNode(attribute3_attr2);
					QDomAttr attribute3_attr3 = xml.createAttribute("type");
						attribute3_attr3.setValue("int");
						attribute3.setAttributeNode(attribute3_attr3);
					attributes.appendChild(attribute3);
				QDomElement attribute4 = xml.createElement("attribute");
					QDomAttr attribute4_attr1 = xml.createAttribute("id");
						attribute4_attr1.setValue("hashtags");
						attribute4.setAttributeNode(attribute4_attr1);
					QDomAttr attribute4_attr2 = xml.createAttribute("title");
						attribute4_attr2.setValue("Hashtags");
						attribute4.setAttributeNode(attribute4_attr2);
					QDomAttr attribute4_attr3 = xml.createAttribute("type");
						attribute4_attr3.setValue("liststring");
						attribute4.setAttributeNode(attribute4_attr3);
					QDomNode options = xml.createElement("options");
						/*
						QString hashtags_list_str;
						for(HashtagSet::Iterator it = hashtags.begin(); it != hashtags.end(); it++)
							hashtags_list_str.append((*it));
						*/
						options.appendChild(xml.createTextNode(hashtags.toLimitedString()));
						attribute4.appendChild(options);
					attributes.appendChild(attribute4);
				graph.appendChild(attributes);
			QDomElement nodes = xml.createElement("nodes");

			for(NodeSet::Iterator it = nodeSet.begin(); it != nodeSet.end(); it++)
			{
				QDomElement node = xml.createElement("node");
					QDomAttr idNode = xml.createAttribute("id");
						idNode.setValue(QString::number((*it)->id));
						node.setAttributeNode(idNode);
					QDomAttr label = xml.createAttribute("label");
					label.setValue((*it)->screen_name);
						node.setAttributeNode(label);
					QDomAttr startNode = xml.createAttribute("start");
					startNode.setValue((*it)->start.toString("yyyy-MM-ddThh:mm:ss"));
						node.setAttributeNode(startNode);
					QDomElement attvalues = xml.createElement("attvalues");
						QDomElement attvalue1 = xml.createElement("attvalue");
							QDomAttr for1 = xml.createAttribute("for");
								for1.setValue("name");
								attvalue1.setAttributeNode(for1);
							QDomAttr value1 = xml.createAttribute("value");
								value1.setValue((*it)->name);
								attvalue1.setAttributeNode(value1);
							attvalues.appendChild(attvalue1);
						QDomElement attvalue2 = xml.createElement("attvalue");
							QDomAttr for2 = xml.createAttribute("for");
								for2.setValue("screen_name");
								attvalue2.setAttributeNode(for2);
							QDomAttr value2 = xml.createAttribute("value");
								value2.setValue((*it)->screen_name);
								attvalue2.setAttributeNode(value2);
							attvalues.appendChild(attvalue2);
						QDomElement attvalue3 = xml.createElement("attvalue");
							QDomAttr for3 = xml.createAttribute("for");
								for3.setValue("tweet_number");
								attvalue3.setAttributeNode(for3);
							QDomAttr value3 = xml.createAttribute("value");
								value3.setValue(QString::number((*it)->weight));
								attvalue3.setAttributeNode(value3);
							attvalues.appendChild(attvalue3);

						QDomElement attvalue4 = xml.createElement("attvalue");
							QDomAttr for4 = xml.createAttribute("for");
								for4.setValue("hashtags");
								attvalue4.setAttributeNode(for4);
							QDomAttr value4 = xml.createAttribute("value");
								/*
								QString hashtags;
								for(HashtagSet::iterator hash_it = (*it)->hashtags.begin(); hash_it != (*it)->hashtags.end(); hash_it++)
								{
									hashtags.append((*hash_it));
								}
								*/
							value4.setValue((*it)->hashtags.toLimitedString());
								attvalue4.setAttributeNode(value4);
							attvalues.appendChild(attvalue4);


						node.appendChild(attvalues);
					nodes.appendChild(node);
			}
				graph.appendChild(nodes);
			QDomElement edges = xml.createElement("edges");

			int i=0;
			for(EdgeSet::Iterator it = edgeSet.begin(); it != edgeSet.end(); it++)
			{
				QDomElement edge = xml.createElement("edge");
					QDomAttr idEdge = xml.createAttribute("id");
						idEdge.setValue(QString::number(i++));
						edge.setAttributeNode(idEdge);
					QDomAttr source = xml.createAttribute("source");
					source.setValue(QString::number((*it)->source));
						edge.setAttributeNode(source);
					QDomAttr target = xml.createAttribute("target");
					target.setValue(QString::number((*it)->target));
						edge.setAttributeNode(target);
					QDomAttr startEdge = xml.createAttribute("start");
						startEdge.setValue((*it)->start.toString("yyyy-MM-ddThh:mm:ss"));
						edge.setAttributeNode(startEdge);
					edges.appendChild(edge);
			}
				graph.appendChild(edges);
			gexf.appendChild(graph);

	QString filePath = path+QDateTime::currentDateTime().toString("/yyyy-MM-ddThh-mm-ss.gexf");
	QFile graphFile(filePath);
	if(!graphFile.open(QIODevice::WriteOnly | QIODevice::Text))
	{
		QMessageBox::critical(this, "Error", "Error writting gexf");
		qApp->quit();
	}
	QTextStream out(&graphFile);
	out << xml.toString(4);
	graphFile.close();
	QMessageBox::information(this, "Graph rendering completed", "Graph file successfully written:<br/><a href=\""+filePath+"\">"+filePath+"</a>");
}
