#include "apidatasift.h"

/*
 * ################################################################################################
 * #                                        StreamUpdater                                         #
 * ################################################################################################
 */


StreamUpdater::StreamUpdater(QObject *parent) : QObject(parent)
{
	_manager = new QNetworkAccessManager(this);
}

StreamUpdater::~StreamUpdater()
{
	delete _manager;
}

void StreamUpdater::getRequest(QNetworkRequest request)
{
	reply = _manager->get(request);
	QObject::connect(reply,SIGNAL(finished()),this,SLOT(requestHandler()));
}

void StreamUpdater::requestHandler()
{
	emit requestReady(this);
}

/*
 * ################################################################################################
 * #                                         ApiDatasift                                          #
 * ################################################################################################
 */

ApiDatasift::ApiDatasift(Graph **graph_ptr, Setting *settings)
{
	ApiDatasift(0, graph_ptr, settings);
}

ApiDatasift::ApiDatasift(QObject *parent, Graph **graph_ptr,  Setting *settings) : QObject(parent),
																				_tweetNumber(0),
																				_refreshing(false),
																				_interactionId(""),
																				_hash(""),
																				_graph_ptr(graph_ptr),
																				_settings(settings)
{
}

ApiDatasift::~ApiDatasift()
{
}

/*
 *	###########################################################################
 *  #                                 Methods                                 #
 *  ###########################################################################
 */

void ApiDatasift::setHash(QString hash)
{
	_settings->logWrite("Hash set to "+hash);
	_hash = hash;
	_interactionId = "";
}

QNetworkRequest ApiDatasift::csdlRequest(QString & csdl)
{
	return QNetworkRequest(QUrl("http://api.datasift.com/compile?csdl="+csdl+
								"&username="+_settings->datasift->id()+
								"&api_key="+_settings->datasift->psswd()));
}

QNetworkRequest ApiDatasift::streamRequest()
{
	return QNetworkRequest(QUrl("http://api.datasift.com/stream?hash="+_hash+
								"&username="+_settings->datasift->id()+
								"&api_key="+_settings->datasift->psswd()+
								"&interaction_id="+_interactionId));
}

int ApiDatasift::tweetNumber()
{
	return _tweetNumber;
}

/*
void ApiDatasift::errorDisplay(QString errorStr)
{
	emit errorSignal();
	QMessageBox::critical(this, "Error", errorStr);
	emit handledSignal();
}
*/

void ApiDatasift::setCsdl(QString csdl)
{
	if(!_refreshing)
	{
		_settings->logWrite("Getting filter from csdl "+csdl);

		_refreshing = true;
		_csdl = csdl;
		_hash = "";
		_interactionId = "";

		StreamUpdater *streamupdater = new StreamUpdater(this);
		QObject::connect(streamupdater,SIGNAL(requestReady(StreamUpdater *)),this,SLOT(csdlHandler(StreamUpdater *)));
		streamupdater->getRequest(csdlRequest(csdl));
	}
}

void ApiDatasift::csdlHandler(StreamUpdater * streamupdater)
{
	switch(streamupdater->reply->error())
	{
		case QNetworkReply::NoError:
		{
			QJson::Parser parser;
			bool parsed;
			QVariantMap settings = parser.parse(streamupdater->reply, &parsed).toMap();

//			if(!parsed) errorDisplay("JSON parsing error");

			setHash(settings["hash"].toString());
			_refreshing = false;
			break;
		}
		case QNetworkReply::AuthenticationRequiredError:
		{
			_settings->datasift->next();
			_refreshing = false;
			setCsdl(_csdl);
			break;
		}
		default:
		{
			qDebug() << "Error" << streamupdater->reply->error() << "getting csdl:";
			qDebug() << streamupdater->reply->errorString();
			qApp->exit(1);
		}
//		default:
//			errorDisplay("NETWORK ERROR :<br/><em>" + reply->errorString() + "</em>");
	}
	streamupdater->deleteLater();
}



void ApiDatasift::refresh()
{
	if(_hash == "")
	{
		_settings->logWrite("Stream hash is empty");
//		errorDisplay("Stream hash is empty");
	}
	else if(!_refreshing)
	{
		_settings->logWrite("refreshing stream");

		_refreshing = true;

		StreamUpdater *streamupdater = new StreamUpdater(this);
		QObject::connect(streamupdater,SIGNAL(requestReady(StreamUpdater *)),this,SLOT(refreshHandler(StreamUpdater *)));
		streamupdater->getRequest(streamRequest());
	}
}



void ApiDatasift::refreshHandler(StreamUpdater * streamupdater)
{
	switch(streamupdater->reply->error())
	{
		case QNetworkReply::NoError:
		{
			QJson::Parser parser;
			bool parsed;
			QVariantMap stream = parser.parse(streamupdater->reply, &parsed).toMap();
			if(parsed)
			{
				foreach (QVariant tweet, stream["stream"].toList())
				{
					_tweetNumber++;
					if((*_graph_ptr) != 0)
					{
						Node *node = nodeFromTweet(tweet.toMap());
						if ((*_graph_ptr)->addNode(node))
							emit getFriends(node->id);

						(*_graph_ptr)->addEdge(edgeFromReply(tweet.toMap()));
						(*_graph_ptr)->addEdge(edgeFromRetweet(tweet.toMap()));
					}
					_interactionId = (tweet.toMap()["interaction"].toMap())["id"].toString();
				}
			}
			else
			{
				qDebug() << "Error parsing datasift response:";
				qDebug() << streamupdater->reply->readAll();
				qApp->exit(1);
			}
			_refreshing = false;
			break;
		}
		case QNetworkReply::AuthenticationRequiredError:
		{
			_settings->logWrite("Datasift account "+_settings->datasift->id()+" depleted");
			_settings->datasift->next();
			_refreshing = false;
			refresh();
			break;
		}
		default:
		{
			qDebug() << "Error" << streamupdater->reply->error() << "refreshing stream:";
			qDebug() << streamupdater->reply->errorString();
			qApp->exit(1);
		}
		//			errorDisplay("NETWORK ERROR :<br/><em>" + reply->errorString() + "</em>");
	}
	streamupdater->deleteLater();
}




