#include "apitwitter2.h"



/*
 * ################################################################################################
 * #                                         TokenFetcher                                         #
 * ################################################################################################
 */

TokenFetcher::TokenFetcher(QObject *parent) : QObject(parent)
{
	_oauthRequest = new KQOAuthRequest;
	_oauthManager = new KQOAuthManager(this);
}

TokenFetcher::~TokenFetcher()
{
	delete _oauthRequest;
	delete _oauthManager;
}

void TokenFetcher::getAccess()
{
	connect(_oauthManager, SIGNAL(temporaryTokenReceived(QString,QString)), this, SLOT(onTemporaryTokenReceived(QString, QString)));
	connect(_oauthManager, SIGNAL(authorizationReceived(QString,QString)), this, SLOT(onAuthorizationReceived(QString, QString)));
	connect(_oauthManager, SIGNAL(accessTokenReceived(QString,QString)), this, SLOT(onAccessTokenReceived(QString,QString)));

	_oauthRequest->initRequest(KQOAuthRequest::TemporaryCredentials, QUrl("https://api.twitter.com/oauth/request_token"));
	_oauthRequest->setConsumerKey(CONSUMER_KEY);
	_oauthRequest->setConsumerSecretKey(CONSUMER_SECRET_KEY);

	_oauthManager->setHandleUserAuthorization(true);

	_oauthManager->executeRequest(_oauthRequest);
}

void TokenFetcher::onTemporaryTokenReceived(QString token, QString tokenSecret)
{
	qDebug() << "Temporary token received: " << token << tokenSecret;
	if(_oauthManager->lastError() == KQOAuthManager::NoError)
		_oauthManager->getUserAuthorization(QUrl("https://api.twitter.com/oauth/authorize"));
}

void TokenFetcher::onAuthorizationReceived(QString token, QString verifier)
{
	qDebug() << "User authorization received: " << token << verifier;
	_oauthManager->getUserAccessTokens(QUrl("https://api.twitter.com/oauth/access_token"));
	if( _oauthManager->lastError() != KQOAuthManager::NoError)
	{
		qDebug() << "OAuth manQObject *parentager error in " << __FILE__ << "(" << __LINE__ << ")";
		QCoreApplication::exit();
	}
}

void TokenFetcher::onAccessTokenReceived(QString token, QString tokenSecret)
{
	qDebug() << "Access token received: " << token << tokenSecret;
	emit accessTokenReceived(token, tokenSecret);
	this->deleteLater();
}

/*
 *	###########################################################################
 *  #                        ApiTwitter - Constructors                        #
 *  ###########################################################################
 */

ApiTwitter::ApiTwitter(Graph **graph_ptr, Setting *settings)
{
	ApiTwitter(NULL, graph_ptr, settings);
}

ApiTwitter::ApiTwitter(QWidget *parent, Graph **graph_ptr, Setting *settings) : QWidget(parent), _graph_ptr(graph_ptr), _settings(settings)
{
	_refreshing = false;
}

/*
 *	###########################################################################
 *  #                           ApiTwitter - Slots                            #
 *  ###########################################################################
 */

void ApiTwitter::run()
{
	qDebug() << "run";

	if(!_refreshing)
	{
		_refreshing = true;
		for (NodeSet::iterator it = (*_graph_ptr)->nodeSet.begin(); it != (*_graph_ptr)->nodeSet.end(); it++)
		{
			TwitterUpdater *updater = new TwitterUpdater(this);
			QObject::connect(updater, SIGNAL(gotFriends(TwitterUpdater *)), this, SLOT(handleFriends(TwitterUpdater *)));
			updater->getFriends((*it)->id, -1, _settings->twitter->id(), _settings->twitter->psswd(), _settings->twitter->index());
		}
		_refreshing = false;
	}
}

void ApiTwitter::handleFriends(TwitterUpdater *updater)
{
	//qDebug() << updater->response;

	updater->deleteLater();
}


/*
 *	###########################################################################
 *  #                     TwitterUpdater - Constructors                       #
 *  ###########################################################################
 */


TwitterUpdater::TwitterUpdater(QWidget *parent) : QWidget(parent)
{
	oauthManager = new KQOAuthManager(this);
	oauthRequest = new KQOAuthRequest;
}

TwitterUpdater::~TwitterUpdater()
{
	delete oauthManager;
	delete oauthRequest;
}

/*
 *	###########################################################################
 *  #                        TwitterUpdater - Methods                         #
 *  ###########################################################################
 */

void TwitterUpdater::getFriends(int id, int cursor, QString token, QString tokenSecret, int tokenId)
{
	_id = id;
	_cursor = cursor;
	_tokenId = tokenId;

	oauthRequest->initRequest(KQOAuthRequest::AuthorizedRequest, QUrl("http://api.twitter.com/1/friends/ids.json"));
	oauthRequest->setHttpMethod(KQOAuthRequest::GET);
	oauthRequest->setConsumerKey(CONSUMER_KEY);
	oauthRequest->setConsumerSecretKey(CONSUMER_SECRET_KEY);
	oauthRequest->setToken(token);
	oauthRequest->setTokenSecret(tokenSecret);

	KQOAuthParameters params;
		params.insert("user_id", QString::number(_id));
		params.insert("cursor", QString::number(_cursor));
	oauthRequest->setAdditionalParameters(params);
	oauthManager->executeRequest(oauthRequest);
	QObject::connect(oauthManager, SIGNAL(requestReady(QByteArray)), this, SLOT(responseHandler(QByteArray)));
}

/*
 *	###########################################################################
 *  #                          TwitterUpdater - Slots                         #
 *  ###########################################################################
 */

void TwitterUpdater::responseHandler(QByteArray response)
{
	this->response = response;
	emit gotFriends(this);
}

