#include "timer.h"

int timeToMs(QTime time, int day)
{
	return time.msec()+1000*time.second()+60000*time.minute()+3600000*time.hour()+86400000*day;
}

/*
 *	###############################################################################################
 *	#                                            Timer                                            #
 *	###############################################################################################
 */

Timer::Timer(QObject * parent, int runtime, QObject * target, const char * method, bool singleShot) : QTimer(parent)
{
	_time = runtime;
	_singleShot = singleShot;
	setSingleShot(singleShot);
	QObject::connect(this, SIGNAL(timeout()), target, method);
}

Timer::Timer(QObject * parent, int runtime, QObject * target, const char * method, const char * method_error, bool singleShot) : QTimer(parent)
{
	_time = runtime;
	_singleShot = singleShot;
	setSingleShot(singleShot);
	QObject::connect(this, SIGNAL(timeout()), target, method);
	QObject::connect(parent, SIGNAL(error()), target, method_error);
}

void Timer::launch()
{
	start(_time);
}

/*
 *	###############################################################################################
 *	#                                        TimerManager                                         #
 *	###############################################################################################
 */
TimerManager::TimerManager(QObject *parent) : QObject(parent)
{

}

TimerManager::~TimerManager()
{
	stop();
	for(QVector<Timer*>::iterator it = _timerVect.begin(); it != _timerVect.end(); it++)
		delete (*it);
}

void TimerManager::addTimer(int runtime, QObject * target, const char * method, bool single)
{
	_timerVect.push_back(new Timer(this, runtime, target, method, single));
}

void TimerManager::addTimer(int runtime, QObject * target, const char * method, const char * method_error, bool single)
{
	_timerVect.push_back(new Timer(this, runtime, target, method, method_error, single));
}

void TimerManager::start()
{
	QVector<Timer*>::iterator it;
	for(it = _timerVect.begin(); it != _timerVect.end(); it++)
		(*it)->launch();
}

void TimerManager::stop()
{
	QVector<Timer*>::iterator it;
	for(it = _timerVect.begin(); it != _timerVect.end(); it++)
		(*it)->stop();
}

void TimerManager::error()
{
	emit error();
}

